@@ -1696,7 +1696,7 @@ static bool create_listen_fdset(fd_set **ppset, int **psock_array, int *listen_n
 	/* each interface gets 4 sockets */
 	count *= 4;
 
-	if(count > FD_SETSIZE) {
+	if(count >= FD_SETSIZE) {
 		DEBUG(0,("create_listen_fdset: Too many file descriptors needed (%d). We can \
 only use %d.\n", count, FD_SETSIZE));
 		SAFE_FREE(pset);
@@ -1712,6 +1712,12 @@ only use %d.\n", count, FD_SETSIZE));
 	FD_ZERO(pset);
 
 	/* Add in the lp_socket_address() interface on 137. */
+	if (ClientNMB < 0 || ClientNMB >= FD_SETSIZE) {
+		errno = EBADF;
+		SAFE_FREE(pset);
+		return True;
+	}
+
 	FD_SET(ClientNMB,pset);
 	sock_array[num++] = ClientNMB;
 	*maxfd = MAX( *maxfd, ClientNMB);
@@ -1721,18 +1727,33 @@ only use %d.\n", count, FD_SETSIZE));
 
 	/* Add in the 137 sockets on all the interfaces. */
 	for (subrec = FIRST_SUBNET; subrec; subrec = NEXT_SUBNET_EXCLUDING_UNICAST(subrec)) {
-		FD_SET(subrec->nmb_sock,pset);
-		sock_array[num++] = subrec->nmb_sock;
-		*maxfd = MAX( *maxfd, subrec->nmb_sock);
+		if (subrec->nmb_sock < 0 || subrec->nmb_sock >= FD_SETSIZE) {
+			/* We have to ignore sockets outside FD_SETSIZE. */
+			sock_array[num++] = -1;
+		} else {
+			FD_SET(subrec->nmb_sock,pset);
+			sock_array[num++] = subrec->nmb_sock;
+			*maxfd = MAX( *maxfd, subrec->nmb_sock);
+		}
 
-		sock_array[num++] = subrec->nmb_bcast;
-		if (subrec->nmb_bcast != -1) {
-			FD_SET(subrec->nmb_bcast,pset);
-			*maxfd = MAX( *maxfd, subrec->nmb_bcast);
+		if (subrec->nmb_bcast < 0 || subrec->nmb_bcast >= FD_SETSIZE) {
+			/* We have to ignore sockets outside FD_SETSIZE. */
+			sock_array[num++] = -1;
+		} else {
+			sock_array[num++] = subrec->nmb_bcast;
+			if (subrec->nmb_bcast != -1) {
+				FD_SET(subrec->nmb_bcast,pset);
+				*maxfd = MAX( *maxfd, subrec->nmb_bcast);
+			}
 		}
 	}
 
 	/* Add in the lp_socket_address() interface on 138. */
+	if (ClientDGRAM < 0 || ClientDGRAM >= FD_SETSIZE) {
+		errno = EBADF;
+		SAFE_FREE(pset);
+		return True;
+	}
 	FD_SET(ClientDGRAM,pset);
 	sock_array[num++] = ClientDGRAM;
 	*maxfd = MAX( *maxfd, ClientDGRAM);
@@ -1742,17 +1763,29 @@ only use %d.\n", count, FD_SETSIZE));
 
 	/* Add in the 138 sockets on all the interfaces. */
 	for (subrec = FIRST_SUBNET; subrec; subrec = NEXT_SUBNET_EXCLUDING_UNICAST(subrec)) {
-		FD_SET(subrec->dgram_sock,pset);
-		sock_array[num++] = subrec->dgram_sock;
-		*maxfd = MAX( *maxfd, subrec->dgram_sock);
+		if (subrec->dgram_sock < 0 || subrec->dgram_sock >= FD_SETSIZE) {
+			/* We have to ignore sockets outside FD_SETSIZE. */
+			sock_array[num++] = -1;
+		} else {
+			FD_SET(subrec->dgram_sock,pset);
+			sock_array[num++] = subrec->dgram_sock;
+			*maxfd = MAX( *maxfd, subrec->dgram_sock);
+		}
 
-		sock_array[num++] = subrec->dgram_bcast;
-		if (subrec->dgram_bcast != -1) {
-			FD_SET(subrec->dgram_bcast,pset);
-			*maxfd = MAX( *maxfd, subrec->dgram_bcast);
+		if (subrec->dgram_bcast < 0 || subrec->dgram_bcast >= FD_SETSIZE) {
+			/* We have to ignore sockets outside FD_SETSIZE. */
+			sock_array[num++] = -1;
+		} else {
+			sock_array[num++] = subrec->dgram_bcast;
+			if (subrec->dgram_bcast != -1) {
+				FD_SET(subrec->dgram_bcast,pset);
+				*maxfd = MAX( *maxfd, subrec->dgram_bcast);
+			}
 		}
 	}
 
+	SMB_ASSERT(count == num);
+
 	*listen_number = count;
 
 	SAFE_FREE(*ppset);
@@ -1876,7 +1909,7 @@ bool listen_for_packets(bool run_election)
 
 #ifndef SYNC_DNS
 	dns_fd = asyncdns_fd();
-	if (dns_fd != -1) {
+	if (dns_fd >= 0 && dns_fd < FD_SETSIZE) {
 		FD_SET(dns_fd, &r_fds);
 		maxfd = MAX( maxfd, dns_fd);
 	}
